@@ -11,7 +11,7 @@ use URPM::Resolve;
 use URPM::Signature;
 
 our @ISA = qw(DynaLoader);
-our $VERSION = '4.15.1';
+our $VERSION = '4.29.1';
 
 URPM->bootstrap($VERSION);
 
@@ -32,7 +32,7 @@ sub set_nofatal {
 
 sub packages_providing {
     my ($urpm, $name) = @_;
-    grep { $_ } map { $urpm->{depslist}[$_] } keys %{$urpm->{provides}{$name} || {}};
+    grep { $_ } map { $urpm->{depslist}[$_] } sort { $a <=> $b } keys %{$urpm->{provides}{$name} || {}};
 }
 
 sub packages_obsoleting {
@@ -184,6 +184,14 @@ sub traverse_tag_find {
     $urpm->traverse_tag($tag, [ $name ], $callback);
 }
 
+
+#- this is used when faking a URPM::DB: $urpm can be used as-a $db
+#- (used for urpmi --env)
+sub create_transaction {
+    my ($urpm) = @_;
+    die "Installing is not supported with a fake environment!";
+}
+
 # wrapper around XS functions
 # it handles error cases
 sub _parse_hdlist_or_synthesis {
@@ -298,20 +306,35 @@ and C<URPM::Transaction>.
 
 =head2 The URPM class
 
+=head3 Initialization
+
 =over 4
 
 =item URPM->new()
 
 The constructor creates a new, empty URPM object. It's a blessed hash that
-contains two fields:
+contains three fields:
+
+=over
+
+=item *
 
 B<depslist> is an arrayref containing the list of depending packages (which are
 C<URPM::Package> objects).
 
+=item *
+
+B<obsoletes> is an hashref containing as keys the list of property names
+obsoleted by the URPM object.
+
+=item *
+
 B<provides> is an hashref containing as keys the list of property names
 provided by the URPM object. The associated value is true if the property is
 versioned.
 
+=back
+
 If the constructor is called with the arguments C<< nofatal => 1 >>, various
 fatal error messages are suppressed (file not found in parse_hdlist() and
 parse_synthesis()).
@@ -320,20 +343,25 @@ parse_synthesis()).
 
 Force the re-reading of the RPM configuration files.
 
-=item URPM::ranges_overlap($range1, $range2)
+=back
 
-This utility function compares two version ranges, in order to calculate
-dependencies properly. The ranges have roughly the form
+=head3 Loading packages data
 
-    [<|<=|==|=>|>] [epoch:]version[-release]
+$urpm->{depslist} is loaded when parsing sources (synthesis, hdlists or a plain
+rpm file).
 
-where epoch, version and release are RPM-style version numbers.
+=over
 
 =item $urpm->parse_synthesis($file [, callback => sub {...} ])
 
 This method gets the B<depslist> and the B<provides> from a synthesis file
 and adds them to the URPM object.
 
+Callback signature is callback(C<URPM>, C<URPM::Package>).
+
+The return value is a two-element array containing the first and the last id
+parsed.
+
 =item $urpm->parse_hdlist($file, %options)
 
 This method loads rpm informations from rpm headers contained in an hdlist
@@ -343,6 +371,8 @@ file and adds them to the URPM object. Allowed options are
     callback => sub { ... }
     keep_all_tags => 0 / 1
 
+Callback signature is callback(C<URPM>, C<URPM::Package>).
+
 The return value is a two-element array containing the first and the last id
 parsed.
 
@@ -358,6 +388,23 @@ and adds them to the URPM object. Allowed options are
 If C<keep_all_tags> isn't specified, URPM will drop all memory-consuming tags
 (notably changelogs, filelists, scriptlets).
 
+Callback signature is callback(URPM::Package).
+
+=back
+
+=head3 Searching packages
+
+=over
+
+=item URPM::ranges_overlap($range1, $range2)
+
+This utility function compares two version ranges, in order to calculate
+dependencies properly. The ranges have roughly the form
+
+    [<|<=|==|=>|>] [epoch:]version[-release]
+
+where epoch, version and release are RPM-style version numbers.
+
 =item $urpm->packages_providing($name)
 
 Returns a list of C<URPM::Package> providing <$name>
@@ -371,17 +418,29 @@ Returns a list of C<URPM::Package> corresponding to the wanted <$name>
 Search an RPM by name or by part of name in the list of RPMs represented by
 this $urpm. The behaviour of the search is influenced by several options:
 
-    strict_name => 0 / 1
-    strict_fullname => 0 / 1
-    src => 0 / 1
+=over
+
+=item  strict_name only match short name (N) =>  0 / 1
+
+=item strict_fullname only match fullname (NVRA) (fast) => 0 / 1
+
+=item src => look only for srpms => 0 / 1
+
+=back
+
+=back
+
+=head3 Debuging
+
+These are used when faking a URPM::DB: $urpm can be used as-a $db
+
+=over
 
 =item $urpm->traverse($callback)
 
 Executes the callback for each package in the depslist, passing a
 C<URPM::Package> object as argument the callback.
 
-This is used when faking a URPM::DB: $urpm can be used as-a $db
-
 =item $urpm->traverse_tag($tag, $names, $callback)
 
 $tag may be one of C<name>, C<whatprovides>, C<whatrequires>, C<whatconflicts>,
@@ -390,7 +449,7 @@ $names is a reference to an array, holding the acceptable values of the said
 tag for the searched variables.
 Then, $callback is called for each matching package in the depslist.
 
-This is used when faking a URPM::DB: $urpm can be used as-a $db
+Callback signature is callback(URPM::Package).
 
 =item $urpm->traverse_tag_find($tag,$name,$callback)
 
@@ -398,7 +457,13 @@ Quite similar to C<traverse_tag>, but stops when $callback returns true.
 
 (also note that only one $name is handled)
 
-This is used when faking a URPM::DB: $urpm can be used as-a $db
+Callback signature is callback(URPM::Package).
+
+=back
+
+=head3 Checking packages
+
+=over
 
 =item URPM::verify_rpm($file, %options)
 
@@ -424,13 +489,6 @@ Imports a key in the RPM database.
     block => '...'
     filename => '...'
 
-=item URPM::spec2srcheader($specfile)
-
-Returns a URPM::Package object containing the header of the source rpm produced
-by the evaluation of the specfile whose path is given as argument. All
-dependencies stored in this header are exactly the one needed to build the
-specfile.
-
 =back
 
 =head2 The URPM::DB class
@@ -452,11 +510,17 @@ the RPM DB should be open in read/write mode.
 
 Rebuilds the RPM database (like C<rpm --rebuilddb>). $prefix defaults to C<"">.
 
+=item verify($prefix)
+
+Verify the RPM database (like C<rpmdb --verify>). $prefix defaults to C<"">.
+
 =item $db->traverse($callback)
 
 Executes the specified callback (a code reference) for each package
 in the DB, passing a C<URPM::Package> object as argument the callback.
 
+Returns the number of packages seen (all).
+
 =item $db->traverse_tag($tag,$names,$callback)
 
 $tag may be one of C<name>, C<whatprovides>, C<whatrequires>, C<whatconflicts>,
@@ -465,12 +529,20 @@ $names is a reference to an array, holding the acceptable values of the said
 tag for the searched variables.
 Then, $callback is called for each matching package in the DB.
 
+Callback signature is callback(URPM::Package).
+
+Returns the number of packages seen (all those that matched provided names).
+
 =item $db->traverse_tag_find($tag,$name,$callback)
 
 Quite similar to C<traverse_tag>, but stops when $callback returns true.
 
 (also note that only one $name is handled)
 
+Callback signature is callback(URPM::Package).
+
+Returns whether callback returned true once.
+
 =item $db->create_transaction()
 
 Creates and returns a new transaction (an C<URPM::Transaction> object) on the
@@ -480,6 +552,30 @@ specified DB.
 
 =head2 The URPM::Package class
 
+=head3 Getting a URPM::Package object
+
+URPM::Package objects are usually retrieved from $urpm->{depslist} after
+having loaded either the RPM DB and/or synthesis files.
+
+It's also possible to get such an object with:
+
+=over
+
+=item URPM::spec2srcheader($specfile)
+
+Returns a URPM::Package object containing the header of the source rpm produced
+by the evaluation of the specfile whose path is given as argument. All
+dependencies stored in this header are exactly the one needed to build the
+specfile.
+
+=item URPM::stream2header($fp)
+
+Returns a URPM::Package object containing the header read from $fp.
+
+=back
+
+=head3 Methods
+
 Most methods of C<URPM::Package> are accessors for the various properties
 of an RPM package.
 
@@ -663,6 +759,8 @@ Full requires tags
 
 Just the required package name.
 
+=item $package->suggests()
+
 =item $package->rflags()
 
 =item $package->filesize()
@@ -751,10 +849,12 @@ list context, returns an array of problems (an empty array indicates success).
 Runs the transaction.
 
 $data is an arbitrary user-provided piece of data to be passed to callbacks.
+It's usually the $urpm object.
 
 Recognized options are:
 
     callback_close  => sub { ... }
+    callback_error  => sub { ... }
     callback_inst   => sub { ... }
     callback_open   => sub { ... }
     callback_trans  => sub { ... }
@@ -771,6 +871,19 @@ Recognized options are:
 
 They roughly correspond to command-line options to rpm(1).
 
+'callback_open' signature is (C<$data>, C<$cb_type>, C<$pkg_id>). It _must_ return a file handler for the asked package.
+
+'callback_close' signature is (C<$data>, C<$cb_type>, C<$pkg_id>). It is called just before URPM close the fd for the installed package.
+
+C<$cb_type> is one of 'open' or 'close'.
+
+Other Callbacks signature is callback(C<$data>, C<$cb_type>, C<$pkg_id>, C<$subtype>, C<$amout>, C<$total>)
+
+C<$cb_type> is one of 'error', 'inst', 'trans' or 'uninst'. C<$subtype> can be 'start', 'progress' or 'stop'.
+For 'error', it can be 'cpio', 'script' or 'unpack'.
+
+The purpose of those callbacks is to report progress (the two last parameters (C<$amount> & C<$total>) enable to compute progress percentage).
+
 =item $trans->traverse($callback)
 
 Executes the specified callback (a code reference) for each package in the
@@ -778,6 +891,28 @@ transaction, passing a C<URPM::Package> object as argument the callback.
 
 =back
 
+=head3 Transaction Element management
+
+=over 4
+
+=item $trans->NElements($fileno)
+
+Returns the number of elements in the transaction.
+
+=item $trans->Element_version($index)
+
+Returns the version of the $index-th element in the transaction.
+
+=item $trans->Element_release($index)
+
+Returns the release of the $index-th element in the transaction.
+
+=item $trans->Element_fullname($index)
+
+Returns the fullname of the $index-th element in the transaction.
+
+=back
+
 =head2 Macro handling functions
 
 =over
@@ -810,6 +945,12 @@ Delete a macro.
 
 Destroys macros.
 
+=back
+
+=head2 Misc other functions
+
+=over
+
 =item setVerbosity($level)
 
 Sets rpm verbosity level. $level is an integer between 2 (RPMMESS_CRIT) and 7
@@ -882,13 +1023,21 @@ B<transaction>: [ { upgrade => [ id ], remove => [ fullname ] } ]
 
 B<transaction_state>: $state object
 
+
+=head1 SEE ALSO
+
+The L<URPM::Resolve> implements the resolving bits.
+
+The L<urpm> package is a higher level module used by the urpmi command line tool,
+the rpmdrake GUI and the drakx installer.
+
 =head1 COPYRIGHT
 
 Copyright 2002, 2003, 2004, 2005 MandrakeSoft SA
 
 Copyright 2005, 2006, 2007, 2008 Mandriva SA
 
-Copyright 2011, 2012, Mangeia
+Copyright 2011, 2012, 2013 Mageia
 
 FranE<ccedil>ois Pons (original author), Rafael Garcia-Suarez, Pixel, Thierry Vignaud <tv@mageia.org> (current maintainer)
 
